@@ -233,6 +233,11 @@ bool GdbDriver::startup(QString cmdStr)
 	"set confirm off\n"
 	"set print static-members off\n"
 	"set print asm-demangle on\n"
+	/*
+	 * Sometimes, gdb prints [New Thread ...] during 'info threads';
+	 * we will not look at thread events anyway, so turn them off.
+	 */
+	"set print thread-events off\n"
 	/*
 	 * Don't assume that program functions invoked from a watch expression
 	 * always succeed.
@@ -1032,6 +1037,10 @@ static bool isNumberish(const char ch)
 
 void skipString(const char*& p)
 {
+    // wchar_t strings begin with L
+    if (*p == 'L')
+	++p;
+
 moreStrings:
     // opening quote
     char quote = *p++;
@@ -1066,7 +1075,7 @@ moreStrings:
 	    }
 	}
     }
-    // is the string continued?
+    // Is the string continued? If so, there is no L in wchar_t strings
     if (*p == ',')
     {
 	// look ahead for another quote
@@ -1326,6 +1335,10 @@ repeat:
 	    checkMultiPart = *p == '\'';
 	    // found a string
 	    skipString(p);
+	} else if (*p == 'L' && (p[1] == '"' || p[1] == '\'')) {
+	    // ditto for wchar_t strings
+	    checkMultiPart = p[1] == '\'';
+	    skipString(p);
 	} else if (*p == '&') {
 	    // function pointer
 	    p++;
@@ -1353,12 +1366,17 @@ repeat:
 		p = e+1;
 	    }
 	} else {
+	moreEnum:
 	    // must be an enumeration value
 	    skipName(p);
-	    // hmm, not necessarily: nan (floating point Not a Number)
-	    // is followed by a number in ()
-	    if (*p == '(')
+	    // nan (floating point Not a Number) is followed by a number in ()
+	    // enum values can look like A::(anonymous namespace)::blue
+	    if (*p == '(') {
+		bool isAnonNS = strncmp(p+1, "anonymous namespace)", 20) == 0;
 		skipNested(p, '(', ')');
+		if (isAnonNS)
+		    goto moreEnum;
+	    }
 	}
 	variable->m_value += QString::fromLatin1(start, p - start);
 
@@ -1375,6 +1393,8 @@ repeat:
 	    
 	    if (*p == '"' || *p == '\'') {
 		skipString(p);
+	    } else if (*p == 'L' && (p[1] == '"' || p[1] == '\'')) {
+		skipString(p);	// wchar_t string
 	    } else if (*p == '<') {
 		// if this value is part of an array, it might be followed
 		// by <repeats 15 times>, which we don't skip here
@@ -1877,11 +1897,14 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 std::list<ThreadInfo> GdbDriver::parseThreadList(const char* output)
 {
     std::list<ThreadInfo> threads;
-    if (strcmp(output, "\n") == 0 || strncmp(output, "No stack.", 9) == 0) {
+    if (strcmp(output, "\n") == 0 ||
+	strncmp(output, "No stack.", 9) == 0 ||
+	strncmp(output, "No threads.", 11) == 0) {
 	// no threads
 	return threads;
     }
 
+    bool newFormat = false;
     const char* p = output;
     while (*p != '\0') {
 	ThreadInfo thr;
@@ -1889,6 +1912,16 @@ std::list<ThreadInfo> GdbDriver::parseThreadList(const char* output)
 	thr.hasFocus = false;
 	while (isspace(*p))		/* may be \n from prev line: see "No stack" below */
 	    p++;
+
+	// recent GDBs write a header line; skip it
+	if (threads.empty() && strncmp(p, "Id   Target", 11) == 0) {
+	    p = strchr(p, '\n');
+	    if (p == NULL)
+		break;
+	    newFormat = true;
+	    continue;	// next line please, '\n' is skipped above
+	}
+
 	if (*p == '*') {
 	    thr.hasFocus = true;
 	    p++;
@@ -1909,12 +1942,47 @@ std::list<ThreadInfo> GdbDriver::parseThreadList(const char* output)
 	    p++;
 
 	/*
-	 * Now follows the thread's SYSTAG. It is terminated by two blanks.
+	 * Now follows the thread's SYSTAG.
 	 */
-	end = strstr(p, "  ");
-	if (end == 0) {
-	    // syntax error; bail out
-	    return threads;
+	if (!newFormat) {
+	    // In the old format, it is terminated by two blanks.
+	    end = strstr(p, "  ");
+	    if (end == 0) {
+		// syntax error; bail out
+		return threads;
+	    }
+	} else {
+	    // In the new format lies crazyness: there is no definitive
+	    // end marker. At best we can guess when the SYSTAG ends.
+	    // A typical thread list on Linux looks like this:
+	    //
+	    //   Id   Target Id         Frame
+	    //   2    Thread 0x7ffff7854700 (LWP 10827) "thrserver" 0x00007ffff7928631 in clone () from /lib64/libc.so.6
+	    // * 1    Thread 0x7ffff7fcc700 (LWP 10808) "thrserver" main () at thrserver.c:84
+	    //
+	    // Looking at GDB's code, the Target Id ends in tokens that
+	    // are bracketed by parentheses or quotes. Therefore,
+	    // we skip (at most) two tokens ('Thread' and the address),
+	    // and then all parts that are in parentheses or quotes.
+	    int n = 0;
+	    end = p;
+	    while (*end) {
+		if (*end == '"') {
+		    skipString(end);
+		    n = 2;
+		} else if (*end == '(') {
+		    skipNested(end, '(', ')');
+		    n = 2;
+		} else if (n < 2) {
+		    while (*end && !isspace(*end))
+			++end;
+		    ++n;
+		} else {
+		    break;
+		}
+		while (isspace(*end))
+		    ++end;
+	    }
 	}
 	thr.threadName = QString::fromLatin1(p, end-p);
 	p = end+2;
@@ -2130,7 +2198,10 @@ uint GdbDriver::parseProgramStopped(const char* output, QString& message)
     // "Program terminated with wignal SIGSEGV"
     // "Program received signal SIGINT" or other signal
     // "Breakpoint..."
-    
+    // GDB since 7.3 prints
+    // "[Inferior 1 (process 13400) exited normally]"
+    // "[Inferior 1 (process 14698) exited with code 01]"
+
     // go through the output, line by line, checking what we have
     const char* start = output - 1;
     uint flags = SFprogramActive;
@@ -2159,6 +2230,26 @@ uint GdbDriver::parseProgramStopped(const char* output, QString& message)
 	    if (endOfMessage == 0)
 		endOfMessage = start + strlen(start);
 	    message = QString::fromLatin1(start, endOfMessage-start);
+	} else if (strncmp(start, "[Inferior ", 10) == 0) {
+	    const char* p = start + 10;
+	    // skip number and space
+	    while (*p && !isspace(*p))
+		++p;
+	    while (isspace(*p))
+		++p;
+	    if (*p == '(') {
+		skipNested(p, '(', ')');
+		if (strncmp(p, " exited ", 8) == 0) {
+		    flags &= ~SFprogramActive;
+
+		    // set message
+		    const char* end = strchr(p, '\n');
+		    if (end == 0)
+			end = p + strlen(p);
+		    // strip [] from the message
+		    message = QString::fromLatin1(start+1, end-start-2);
+		}
+	    }
 	} else if (strncmp(start, "Breakpoint ", 11) == 0) {
 	    /*
 	     * We stopped at a (permanent) breakpoint (gdb doesn't tell us
