@@ -4,18 +4,16 @@
  * See the file COPYING in the toplevel directory of the source directory.
  */
 
-#include <qheader.h>
-#include <kglobalsettings.h>
-#include <klocale.h>			/* i18n */
-#include <kiconloader.h>
-#include <qfontdialog.h>
-#include <qmessagebox.h>
-#include <qpopupmenu.h>
-#include <qregexp.h>
-#include <qstringlist.h>
-#include <stdlib.h>			/* strtoul */
 #include "regwnd.h"
 #include "dbgdriver.h"
+#include <kglobalsettings.h>
+#include <klocale.h>			/* i18n */
+#include <QMenu>
+#include <QRegExp>
+#include <QStringList>
+#include <QHeaderView>
+#include <QContextMenuEvent>
+#include <stdlib.h>			/* strtoul */
 
 /** 
  * Register display modes
@@ -94,11 +92,11 @@ uint RegisterDisplay::bitMap[] = {
   64, 80, 128, /*default*/32,
 };
 
-class ModeItem : public QListViewItem 
+class ModeItem : public QTreeWidgetItem
 {
 public:
-    ModeItem(QListView* parent, const QString& name) : QListViewItem(parent, name) {}
-    ModeItem(QListViewItem* parent) : QListViewItem(parent) {}
+    ModeItem(QTreeWidget* parent, const QString& name) : QTreeWidgetItem(parent, QStringList(name)) {}
+    ModeItem(QTreeWidgetItem* parent) : QTreeWidgetItem(parent) {}
 
     virtual void setMode(RegisterDisplay mode) = 0;
     virtual RegisterDisplay mode() = 0;
@@ -112,22 +110,27 @@ public:
 		     RegisterDisplay mode) :
 	ModeItem(parent, name), matcher(pattern), gmode(mode)
     {
-        setExpandable(true);
-        setOpen(true);
+	setExpanded(true);
     } 
+
     bool matchName(const QString& str) const
     {
         return matcher.exactMatch(str);
     }
+
     virtual void setMode(RegisterDisplay mode)
     {
 	gmode=mode; 
-	QListViewItem *it=firstChild();
-	for (; 0!=it; it=it->nextSibling()) {
-	    (static_cast<ModeItem*>(it))->setMode(gmode);
+	for(int i = 0; i < childCount(); i++)
+	{
+	    static_cast<ModeItem*>(child(i))->setMode(gmode);
 	}
     }
-    virtual RegisterDisplay mode() { return gmode; }
+
+    virtual RegisterDisplay mode()
+    {
+        return gmode;
+    }
 
 private:
     QRegExp matcher;
@@ -148,11 +151,6 @@ public:
     RegisterDisplay m_mode;		/* display mode */
     bool m_changes;
     bool m_found;
-
-protected:
-    virtual void paintCell(QPainter*, const QColorGroup& cg,
-			   int column, int width, int alignment);
-
 };
 
 
@@ -201,8 +199,8 @@ static QString toBinary(QString hex)
     };
     QString result;
     
-    for (unsigned i = 2; i < hex.length(); i++) {
-	int idx = hexCharToDigit(hex[i].latin1());
+    for (int i = 2; i < hex.length(); i++) {
+	int idx = hexCharToDigit(hex[i].toLatin1());
 	if (idx < 0) {
 	    // not a hex digit; no conversion
 	    return hex;
@@ -211,7 +209,7 @@ static QString toBinary(QString hex)
 	result += bindigits;
     }
     // remove leading zeros
-    switch (hexCharToDigit(hex[2].latin1())) {
+    switch (hexCharToDigit(hex[2].toLatin1())) {
     case 0: case 1: result.remove(0, 3); break;
     case 2: case 3: result.remove(0, 2); break;
     case 4: case 5:
@@ -226,7 +224,7 @@ static QString toOctal(QString hex)
     int shift = 0;
     unsigned v = 0;
     for (int i = hex.length()-1; i >= 2; i--) {
-	int idx = hexCharToDigit(hex[i].latin1());
+	int idx = hexCharToDigit(hex[i].toLatin1());
 	if (idx < 0)
 	    return hex;
 	v += idx << shift;
@@ -251,13 +249,12 @@ static QString toDecimal(QString hex)
      * We convert only numbers that are small enough for this computer's
      * size of long integers.
      */
-    if (hex.length() > sizeof(unsigned long)*2+2)	/*  count in leading "0x" */
+    if (hex.length() > int(sizeof(unsigned long)*2+2))	/*  count in leading "0x" */
 	return hex;
 
-    const char* start = hex.latin1();
-    char* end;
-    unsigned long val = strtoul(start, &end, 0);
-    if (start == end)
+    bool ok = false;
+    unsigned long val = hex.toULong(&ok, 0);
+    if (!ok)
 	return hex;
     else
 	return QString().setNum(val);
@@ -282,16 +279,20 @@ static char* toRaw(const QString& hex, uint& length)
 	uint j=0;
 	if (hex.length()<=2) return 0;
 	for (int i=hex.length()-1; i>=2; ) {
-	    if (j%2==0) data[j/2]=hexCharToDigit(hex[i].latin1());
-	    else data[j/2]|=(hexCharToDigit(hex[i].latin1())<<4);
+	    if (j%2==0)
+		data[j/2]=hexCharToDigit(hex[i].toLatin1());
+	    else
+		data[j/2]|=(hexCharToDigit(hex[i].toLatin1())<<4);
 	    i--;j++;
 	}
     } else { // big endian
 	uint j=0;
 	if (hex.length()<=2) return 0;
-	for (uint i=2; i<hex.length(); ) {
-	    if (j%2==0) data[j/2]=hexCharToDigit(hex[i].latin1())<<4;
-	    else data[j/2]|=hexCharToDigit(hex[i].latin1());
+	for (int i=2; i<hex.length(); ) {
+	    if (j%2==0)
+		data[j/2]=hexCharToDigit(hex[i].toLatin1())<<4;
+	    else
+		data[j/2]|=hexCharToDigit(hex[i].toLatin1());
 	    i++;j++;
 	}
     }
@@ -320,12 +321,16 @@ static long double extractNumber(const QString& hex)
 static QString toFloat(const QString& hex, char p)
 {
     uint bits;
-    uint prec=6;
+    int prec=6;
     if (hex.length()<=10) { bits=32; prec=6; }
     else if (hex.length()<=18) { bits=64; prec=17; }
     else { bits=80; prec=20; }
 
-    QString cooked=QString::number(extractNumber(hex), p, prec);
+    char fmt[8] = "%.*Lf";
+    fmt[4] = p;
+    char buf[50];
+    sprintf(buf, fmt, prec, extractNumber(hex));
+    QString cooked = QString::fromLatin1(buf);
     if (p=='e') {
 	prec+=7;    
 	while (cooked.length()<prec) cooked=cooked.prepend(" ");
@@ -351,7 +356,7 @@ static QString convertSingle(const QString& raw, const RegisterDisplay mode)
 QString convertRaw(const RegisterInfo reg, RegisterDisplay mode)
 {
     QString cooked;
-    uint totalNibles=0, nibles=mode.bits()>>2;
+    int totalNibles=0, nibles=mode.bits()>>2;
     if (RegisterDisplay::nada!=mode.presentationFlag() &&
         reg.rawValue.length() > 2 && reg.rawValue[0] == '0' && reg.rawValue[1] == 'x')
     {
@@ -379,7 +384,7 @@ QString convertRaw(const RegisterInfo reg, RegisterDisplay mode)
     {
 	cooked = reg.cookedValue;
     }
-    if (cooked.at(0)!=' ' && cooked.at(0)!='-' && cooked.at(0)!='+')
+    if (!cooked.isEmpty() && cooked.at(0) != ' ' && cooked.at(0) != '-' && cooked.at(0) != '+')
 	cooked.prepend(" ");
     return cooked;
 }
@@ -401,72 +406,51 @@ void RegisterViewItem::setMode(RegisterDisplay mode)
     setText(2, cookedValue);
 }
 
-void RegisterViewItem::paintCell(QPainter* p, const QColorGroup& cg,
-				 int column, int width, int alignment)
-{
-    if (m_changes) {
-	QColorGroup newcg = cg;
-	newcg.setColor(QColorGroup::Text, red);
-	QListViewItem::paintCell(p, newcg, column, width, alignment);
-    } else {
-	QListViewItem::paintCell(p, cg, column, width, alignment);
-    }
-}
 
-
-RegisterView::RegisterView(QWidget* parent, const char* name) :
-	QListView(parent, name)
+RegisterView::RegisterView(QWidget* parent) :
+	QTreeWidget(parent)
 {
-    setSorting(-1);
     setFont(KGlobalSettings::fixedFont());
 
-    QPixmap iconRegs = UserIcon("regs.xpm");
-    QPixmap iconWatchcoded = UserIcon("watchcoded.xpm");
-    QPixmap iconWatch = UserIcon("watch.xpm");
+    QTreeWidgetItem* header = headerItem();
+    header->setText(0, i18n("Register"));
+    header->setText(1, i18n("Value"));
+    header->setText(2, i18n("Decoded value"));
 
-    addColumn(QIconSet(iconRegs), i18n("Register"));
-    addColumn(QIconSet(iconWatchcoded), i18n("Value"));
-    addColumn(QIconSet(iconWatch), i18n("Decoded value"));
+    setAllColumnsShowFocus(true);
 
-    setColumnAlignment(0,AlignLeft);
-    setColumnAlignment(1,AlignLeft);
-    setColumnAlignment(2,AlignLeft);
-
-    setAllColumnsShowFocus( true );
-    header()->setClickEnabled(false);
-
-    connect(this, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
-	    SLOT(rightButtonClicked(QListViewItem*,const QPoint&,int)));
-
-    m_modemenu = new QPopupMenu(this, "ERROR");
+    m_modemenu = new QMenu("ERROR", this);
     for (uint i=0; i<sizeof(menuitems)/sizeof(MenuPair); i++) {
 	if (menuitems[i].isSeparator())
-	    m_modemenu->insertSeparator();
-	else
-	    m_modemenu->insertItem(i18n(menuitems[i].name), menuitems[i].mode);
+	    m_modemenu->addSeparator();
+	else {
+	    QAction* action = m_modemenu->addAction(i18n(menuitems[i].name));
+	    action->setData(menuitems[i].mode);
+	    action->setCheckable(true);
+	}
     }
-    connect(m_modemenu,SIGNAL(activated(int)),SLOT(slotModeChange(int)));
+    connect(m_modemenu, SIGNAL(triggered(QAction*)), SLOT(slotModeChange(QAction*)));
     
-    new GroupingViewItem(this, "MIPS VU", "^vu.*",
-                         RegisterDisplay::bits32|RegisterDisplay::realE);
-    new GroupingViewItem(this, "AltiVec", "^vr.*",
-                         RegisterDisplay::bits32|RegisterDisplay::realE);
-    new GroupingViewItem(this, "POWER real", "^fpr.*",
-                         RegisterDisplay::bits32|RegisterDisplay::realE);
-    new GroupingViewItem(this, "MMX", "^mm.*", 
-                         RegisterDisplay::bits32|RegisterDisplay::realE);
-    new GroupingViewItem(this, "SSE", "^xmm.*", 
-                         RegisterDisplay::bits32|RegisterDisplay::realE);
-    new GroupingViewItem(this, "x87", "^st.*",
-                         RegisterDisplay::bits80|RegisterDisplay::realE);
-    new GroupingViewItem(this, i18n("x86/x87 segment"), 
-                         "(^cs$|^ss$|^ds$|^es$|^fs$|^gs$|^fiseg$|^foseg$)",
-			 RegisterDisplay::nada);
-    new GroupingViewItem(this, i18n("Flags"), 
-                         "(^eflags$|^fctrl$|^mxcsr$|^cr$|^fpscr$|^vscr$|^ftag$|^fstat$)", 
-                         RegisterDisplay::bits32|RegisterDisplay::binary);
     new GroupingViewItem(this, i18n("GP and others"), "^$",
 			 RegisterDisplay::nada);
+    new GroupingViewItem(this, i18n("Flags"),
+			 "(^eflags$|^fctrl$|^mxcsr$|^cr$|^fpscr$|^vscr$|^ftag$|^fstat$)",
+			 RegisterDisplay::bits32|RegisterDisplay::binary);
+    new GroupingViewItem(this, i18n("x86/x87 segment"),
+			 "(^cs$|^ss$|^ds$|^es$|^fs$|^gs$|^fiseg$|^foseg$)",
+			 RegisterDisplay::nada);
+    new GroupingViewItem(this, "x87", "^st.*",
+			 RegisterDisplay::bits80|RegisterDisplay::realE);
+    new GroupingViewItem(this, "SSE", "^xmm.*",
+			 RegisterDisplay::bits32|RegisterDisplay::realE);
+    new GroupingViewItem(this, "MMX", "^mm.*",
+			 RegisterDisplay::bits32|RegisterDisplay::realE);
+    new GroupingViewItem(this, "POWER real", "^fpr.*",
+			 RegisterDisplay::bits32|RegisterDisplay::realE);
+    new GroupingViewItem(this, "AltiVec", "^vr.*",
+			 RegisterDisplay::bits32|RegisterDisplay::realE);
+    new GroupingViewItem(this, "MIPS VU", "^vu.*",
+			 RegisterDisplay::bits32|RegisterDisplay::realE);
 
     updateGroupVisibility();
     setRootIsDecorated(true);
@@ -480,31 +464,35 @@ RegisterView::~RegisterView()
 
 GroupingViewItem* RegisterView::findMatchingGroup(const QString& regName)
 {
-    for (QListViewItem* it = firstChild(); it != 0; it = it->nextSibling())
+    for (int i = 0; i < topLevelItemCount(); i++)
     {
-	GroupingViewItem* i = static_cast<GroupingViewItem*>(it);
-	if (i->matchName(regName))
-	    return i;
+	GroupingViewItem* it = static_cast<GroupingViewItem*>(topLevelItem(i));
+	if (it->matchName(regName))
+	    return it;
     }
     // not better match found, so return "GP and others"
-    return static_cast<GroupingViewItem*>(firstChild());
+    return static_cast<GroupingViewItem*>(topLevelItem(0));
 }
 
 GroupingViewItem* RegisterView::findGroup(const QString& groupName)
 {
-    for (QListViewItem* it = firstChild(); it != 0; it = it->nextSibling())
+    for (int i = 0; i < topLevelItemCount(); i++)
     {
+	QTreeWidgetItem* it = topLevelItem(i);
 	if (it->text(0) == groupName)
 	    return static_cast<GroupingViewItem*>(it);
     }
+    // return that nothing was found.
     return 0;
 }
 
+// only show a group if it has subitems.
 void RegisterView::updateGroupVisibility()
 {
-    for (QListViewItem* it = firstChild(); it != 0; it = it->nextSibling())
+    for(int i = 0; i < topLevelItemCount(); i++)
     {
-	it->setVisible(it->childCount() > 0);
+	QTreeWidgetItem* item = topLevelItem(i);
+	item->setHidden(item->childCount() == 0);
     }
 }
 
@@ -519,8 +507,7 @@ void RegisterView::updateRegisters(const std::list<RegisterInfo>& regs)
     }
 
     // parse register values
-    // must iterate last to first, since QListView inserts at the top
-    for (std::list<RegisterInfo>::const_reverse_iterator reg = regs.rbegin(); reg != regs.rend(); ++reg)
+    for (std::list<RegisterInfo>::const_iterator reg = regs.begin(); reg != regs.end(); ++reg)
     {
 	// check if this is a new register
 	RegMap::iterator i = m_registers.find(reg->regName);
@@ -534,7 +521,11 @@ void RegisterView::updateRegisters(const std::list<RegisterInfo>& regs)
 	    {
 		it->m_changes = true;
 		it->setValue(*reg);
-		repaintItem(it);
+
+		it->setForeground(0,Qt::red);
+		it->setForeground(1,Qt::red);
+		it->setForeground(2,Qt::red);
+
 	    } else {
 		/*
 		 * If there was a change last time, but not now, we
@@ -542,7 +533,9 @@ void RegisterView::updateRegisters(const std::list<RegisterInfo>& regs)
 		 */
 		if (it->m_changes) {
 		    it->m_changes = false;
-		    repaintItem(it);
+		    it->setForeground(0,Qt::black);
+		    it->setForeground(1,Qt::black);
+		    it->setForeground(2,Qt::black);
 		}
 	    }
 	}
@@ -571,43 +564,47 @@ void RegisterView::updateRegisters(const std::list<RegisterInfo>& regs)
 
     updateGroupVisibility();
     setUpdatesEnabled(true);
-    triggerUpdate();
 }
   
 
-void RegisterView::rightButtonClicked(QListViewItem* item, const QPoint& p, int)
+void RegisterView::contextMenuEvent(QContextMenuEvent* event)
 {
+    QTreeWidgetItem *item = itemAt(event->pos());
+
     if (item) {
         RegisterDisplay mode=static_cast<ModeItem*>(item)->mode();        
-        for (unsigned int i = 0; i<sizeof(menuitems)/sizeof(MenuPair); i++) {
-            m_modemenu->setItemChecked(menuitems[i].mode, 
-                                       mode.contains(menuitems[i].mode));
+	int i = 0;
+	foreach(QAction* action, m_modemenu->actions())
+	{
+	    action->setChecked(mode.contains(menuitems[i].mode));
+	    ++i;
         }
-        m_modemenu->setCaption(item->text(0));
-        m_modemenu->popup(p);
+        m_modemenu->setTitle(item->text(0));
+	m_modemenu->popup(event->globalPos());
+
+	event->accept();
     }    
 }
 
-void RegisterView::slotModeChange(int pcode)
+void RegisterView::slotModeChange(QAction* action)
 {
-    RegMap::iterator it=m_registers.find(m_modemenu->caption());
+    RegMap::iterator it=m_registers.find(m_modemenu->title());
     ModeItem* view;
     if (it != m_registers.end())
 	view = it->second;
     else
-	view = findGroup(m_modemenu->caption());
+	view = findGroup(m_modemenu->title());
 
     if (view) {
 	RegisterDisplay mode = view->mode();
-	mode.changeFlag(pcode);
+	mode.changeFlag(action->data().toInt());
 	view->setMode(mode);
     }
 }
 
-void RegisterView::paletteChange(const QPalette& oldPal)
+void RegisterView::paletteChange(const QPalette&)
 {
     setFont(KGlobalSettings::fixedFont());
-    QListView::paletteChange(oldPal);
 }
 
 #include "regwnd.moc"
