@@ -6,17 +6,17 @@
 
 #include "xsldbgdriver.h"
 #include "exprwnd.h"
-#include <qstringlist.h>
+#include <QFileInfo>
+#include <QRegExp>
+#include <QStringList>
 #include <klocale.h>            /* i18n */
 #include <ctype.h>
+#include <signal.h>
 #include <stdlib.h>             /* strtol, atoi */
 #include <string.h>             /* strcpy */
 #include <kmessagebox.h>
 
 #include "assert.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "mydebug.h"
 
 
@@ -104,13 +104,8 @@ static XsldbgCmdInfo cmds[] = {
 #define MAX_FMTLEN 200
 
 XsldbgDriver::XsldbgDriver():
-DebuggerDriver(), m_gdbMajor(2), m_gdbMinor(0)
+	DebuggerDriver()
 {
-    m_promptRE.setPattern("\\(xsldbg\\) .*> ");
-    m_promptMinLen = 11;
-    m_promptLastChar = ' ';
-  
-    m_markerRE.setPattern("^Breakpoint for file ");
     m_haveDataFile = FALSE;
 
 #ifndef NDEBUG
@@ -211,26 +206,6 @@ QStringList XsldbgDriver::boolOptionList() const
     return allOptions;
 }
 
-
-void
-XsldbgDriver::slotReceiveOutput(KProcess * process, char *buffer,
-                                int buflen)
-{
-    //TRACE(buffer);
-    if (m_state != DSidle) {
-        //    TRACE(buffer);
-        DebuggerDriver::slotReceiveOutput(process, buffer, buflen);
-    } else {
-        if (strncmp(buffer, "quit", 4) == 0) {
-            TRACE("Ignoring text when xsldbg is quiting");
-        } else {
-            TRACE
-                ("Stray output received by XsldbgDriver::slotReceiveOutput");
-            TRACE(buffer);
-        }
-    }
-}
-
 bool
 XsldbgDriver::startup(QString cmdStr)
 {
@@ -258,41 +233,8 @@ XsldbgDriver::commandFinished(CmdQueueItem * cmd)
         return;
     }
 
-    switch (cmd->m_cmd) {
-        case DCinitialize:
-            // get version number from preamble
-            {
-                int len;
-                QRegExp xsldbgVersion("^XSLDBG [0-9]+\\.[0-9]+\\.[0-9]+");
-                int offset = xsldbgVersion.match(m_output, 0, &len);
-
-                if (offset >= 0) {
-                    char *start = m_output + offset + 7;        // skip "^XSLDBG "
-                    char *end;
-
-                    TRACE("Reading version");
-                    TRACE(start);
-                    m_gdbMajor = strtol(start, &end, 10);
-                    m_gdbMinor = strtol(end + 1, 0, 10);        // skip "."
-                    if (start == end) {
-                        // nothing was parsed
-                        m_gdbMajor = 0;
-                        m_gdbMinor = 7;
-                    }
-                } else {
-                    // assume some default version (what would make sense?)
-                    m_gdbMajor = 0;
-                    m_gdbMinor = 7;
-                }
-                TRACE(QString("Got version ") +
-                      QString::number(m_gdbMajor) + "." +
-                      QString::number(m_gdbMinor));
-                break;
-            }
-        default:;
-    }
     /* ok, the command is ready */
-    emit commandReceived(cmd, m_output);
+    emit commandReceived(cmd, m_output.constData());
 
     switch (cmd->m_cmd) {
         case DCbt:
@@ -302,7 +244,7 @@ XsldbgDriver::commandFinished(CmdQueueItem * cmd)
         case DCstep:
         case DCnext:
         case DCfinish:{
-	  if (!::isErrorExpr(m_output))
+	  if (!::isErrorExpr(m_output.constData()))
             parseMarker();
 	  else{
 	    // This only shows an error for DCinfolocals 
@@ -321,33 +263,45 @@ XsldbgDriver::commandFinished(CmdQueueItem * cmd)
     }
 }
 
+int
+XsldbgDriver::findPrompt(const QByteArray& output) const
+{
+    /*
+     * If there's a prompt string in the collected output, it must be at
+     * the very end. We do a quick check whether the last characters of
+     * output are suitable and do the full search only if they are.
+     */
+    int len = output.length();
+    if (len < 11 || output[len-1] != ' ' || output[len-2] != '>')
+	return -1;
+
+    // There can be text between "(xsldbg) " and the "> " at the end
+    // since we do not know what that text is, we accept the former
+    // anywhere in the output.
+    return output.indexOf("(xsldbg) ");
+}
+
 void
 XsldbgDriver::parseMarker()
 {
+    char *p = m_output.data();
 
-    // TRACE("parseMarker : xsldbg");
-    //  TRACE(m_output);
-    int len, markerStart = -1;
-    char *p = m_output;
-
-    while (markerStart == -1) {
+    for (;;) {
         if ((p == 0) || (*p == '\0')) {
-            m_output[0] = '\0';
+            m_output.clear();
             return;
         }
-        //TRACE(QString("parseMarker is looking at :") + p);
-        markerStart = m_markerRE.match(p, 0, &len);
-        if (markerStart == -1) {
-            // try to marker on next line !
-            p = strchr(p, '\n');
-            if ((p != 0) && (*p != '\0'))
-                p++;
-        }
+        if (strncmp(p, "Breakpoint for file ", 20) == 0)
+	    break;
+	// try to marker on next line !
+	p = strchr(p, '\n');
+	if ((p != 0) && (*p != '\0'))
+	    p++;
     }
 
 
     // extract the marker
-    char *startMarker = p + markerStart + len;
+    char *startMarker = p + 20;
 
     //TRACE(QString("found marker:") + startMarker);
     char *endMarker = strchr(startMarker, '\n');
@@ -358,12 +312,12 @@ XsldbgDriver::parseMarker()
     *endMarker = '\0';
 
     // extract filename and line number
-    static QRegExp MarkerRE(" at line [0-9]+");
+    static QRegExp MarkerRE(" at line (\\d+)");
 
-    int lineNoStart = MarkerRE.match(startMarker, 0, &len);
+    int lineNoStart = MarkerRE.indexIn(startMarker);
 
     if (lineNoStart >= 0) {
-        int lineNo = atoi(startMarker + lineNoStart + 8);
+        int lineNo = MarkerRE.cap(1).toInt();
 
         DbgAddr address;
 
@@ -412,7 +366,7 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg)
     }
 
     QString cmdString;
-    cmdString.sprintf(cmds[cmd].fmt, strArg.latin1());
+    cmdString.sprintf(cmds[cmd].fmt, strArg.toLatin1().constData());
     return cmdString;
 }
 
@@ -514,14 +468,11 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg, int intArg)
         }
         if (cmd == DCinfoline) {
             // must split off file name part
-            int slash = strArg.findRev('/');
-
-            if (slash >= 0)
-                strArg = strArg.right(strArg.length() - slash - 1);
+            strArg = QFileInfo(strArg).fileName();
         }
-        cmdString.sprintf(cmds[cmd].fmt, strArg.latin1(), intArg);
+        cmdString.sprintf(cmds[cmd].fmt, strArg.toLatin1().constData(), intArg);
     } else {
-        cmdString.sprintf(cmds[cmd].fmt, intArg, strArg.latin1());
+        cmdString.sprintf(cmds[cmd].fmt, intArg, strArg.toLatin1().constData());
     }
     return cmdString;
 }
@@ -537,7 +488,9 @@ XsldbgDriver::makeCmdString(DbgCommand cmd, QString strArg1,
     normalizeStringArg(strArg2);
 
     QString cmdString;
-    cmdString.sprintf(cmds[cmd].fmt, strArg1.latin1(), strArg2.latin1());
+    cmdString.sprintf(cmds[cmd].fmt,
+		      strArg1.toLatin1().constData(),
+		      strArg2.toLatin1().constData());
     return cmdString;
 }
 
@@ -640,7 +593,7 @@ XsldbgDriver::terminate()
     qDebug("XsldbgDriver::Terminate");
     flushCommands();
     executeCmdString(DCinitialize, "quit\n", true);
-    kill(SIGTERM);
+    ::kill(pid(), SIGTERM);
     m_state = DSidle;
 }
 
@@ -650,7 +603,7 @@ XsldbgDriver::detachAndTerminate()
     qDebug("XsldbgDriver::detachAndTerminate");
     flushCommands();
     executeCmdString(DCinitialize, "quit\n", true);
-    kill(SIGINT);
+    ::kill(pid(), SIGINT);
 }
 
 void
@@ -659,7 +612,7 @@ XsldbgDriver::interruptInferior()
     // remove accidentally queued commands
     qDebug("interruptInferior");
     flushHiPriQueue();
-    kill(SIGINT);
+    ::kill(pid(), SIGINT);
 }
 
 static bool
@@ -1329,7 +1282,7 @@ XsldbgDriver::parseChangeWD(const char *output, QString & message)
 
     if (strncmp(output, "Change to directory", 20) == 0) {
         output = output + 20;   /* skip 'Change to directory' */
-        message = QString(output).simplifyWhiteSpace();
+        message = QString(output).simplified();
         if (message.isEmpty()) {
             message = i18n("New working directory: ") + m_programWD;
             isGood = true;
@@ -1345,18 +1298,11 @@ XsldbgDriver::parseChangeExecutable(const char *output, QString & message)
     TRACE(QString("XsldbgDriver::parseChangeExecutable :") + output);
     m_haveCoreFile = false;
 
-    /*
-     * The command is successful if there is no output or the single
-     * message (no debugging symbols found)...
-     */
-    QRegExp exp(".*Load of source deferred. Use the run command.*");
-    int len, index = exp.match(output, 0, &len);
-
-    if (index != -1) {
+    if (strstr(output, "Load of source deferred. Use the run command") != 0) {
         TRACE("Parsed stylesheet executable");
-        message = "";
+        message = QString();
     }
-    return (output[0] == '\0') || (index != -1);
+    return message.isEmpty();
 }
 
 bool
@@ -1364,10 +1310,8 @@ XsldbgDriver::parseCoreFile(const char *output)
 {
     TRACE("XsldbgDriver::parseCoreFile");
     TRACE(output);
-    QRegExp exp(".*Load of data file deferred. Use the run command.*");
-    int len, index = exp.match(output, 0, &len);
 
-    if (index != -1) {
+    if (strstr(output, "Load of data file deferred. Use the run command") != 0) {
         m_haveCoreFile = true;
         TRACE("Parsed data file name");
     }
